/*
 * original:lejos-osek/src/nxtvm/platform/nxt/nxt_motor.c rev.1.1.1.1
 * modified history.
 * 05/23/2011: Modifier Takuya Ishikawa
 * 05/25/2010: Modifier Takuya Ishikawa
 *           : Devided file nxt_motor.c -> tRotaryEncoderBody.c and tMotor.c
 *             This file has routines to get and set revolution,
 *             decode rotary encoder, set speed(PWM)
 *           : Modified port number to component attribute
 *           : Added terminate routine which stops motor
 *           : Changed interface name to entry function name with TECS naming rule
 *           : Modified some variables to component variables
 *           : Modified some constants to component attributes
 *           : Modified data type to ITRON spec.
 * 12/13/2009: Modifider Ryosuke Takeuchi
 *           : Modified Device Interface to ITRON-SIL interface and Changed
 *             AT91SAM7.h to SIL defined include file.
 *           : Modified nxt_motor_init for ITRON ATT_INI static API.
 *           : Modified nxt_motor_1kHz_process to callback function.
 */
/*
 * ̃t@C tecsgen ɂev[gƂĎ܂
 * ̃t@CҏWĎgp邱ƂӐ}Ă܂
 * tecsgen ̍Ďsɂ㏑Ă܂߁Aʏ
 *   gen/tMotor_template.c => src/tMotor.c
 * ̂悤ɖO, tH_ύXĂC܂
 */
/* #[<PREAMBLE>]#
 * #[<...>]#  #[</...>]# ň͂܂ꂽRg͕ҏWȂł
 * tecsmerge ɂ}[WɎgp܂
 *
 * ANZX}N #_CAAM_#
 * portNumber       uint8_t          ATTR_portNumber 
 * revolution       int32_t          VAR_revolution  
 * pwm              int8_t           VAR_pwm         
 * preEdge          uint32_t         VAR_preEdge     
 *
 * Ăь֐ #_TCPF_#
 * call port : cAvrMotor  signature: sAvrMotor context: task
 *   void           cAvrMotor_setMotor( uint8_t portNumber, int8_t powerPercent, bool_t brake );
 *
 * #[</PREAMBLE>]# */

#include "tMotor_tecsgen.h"

#ifndef E_OK
#define	E_OK	0		/* success */
#define	E_ID	(-18)	/* illegal ID */
#endif

/*
 *  MA0, MB0, MC0
 */
const uint32_t motorDir[3] = { MOTOR_A0, MOTOR_B0, MOTOR_C0, };

/*
 *  MA1, MB1, MC1
 */
const uint32_t motorEdge[3] = { MOTOR_A1, MOTOR_B1, MOTOR_C1, };


/* 󂯌֐ #_TEPF_# */
/* #[<ENTRY_PORT>]# eMotor
 * entry port: eMotor
 * signature:  sMotor
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eMotor_setPWM
 * name:         eMotor_setPWM
 * global_name:  tMotor_eMotor_setPWM
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eMotor_setPWM(CELLIDX idx, int8_t pwm, bool_t brakeMode)
{
    CELLCB	*p_cellcb;
    if (VALID_IDX(idx)) {
        p_cellcb = GET_CELLCB(idx);
    }
    
    if(pwm > 100){
        pwm = 100;
    } else if(pwm < -100){
        pwm = -100;
    }
    
    /*
     *  brakeMode
     *  0: t[g[h
     *  1: u[L[h
     */
    cAvrMotor_setMotor(ATTR_portNumber, pwm, brakeMode);
}

/* #[<ENTRY_FUNC>]# eMotor_setRevolution
 * name:         eMotor_setRevolution
 * global_name:  tMotor_eMotor_setRevolution
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eMotor_setRevolution(CELLIDX idx, int32_t revolution)
{
    CELLCB	*p_cellcb;
    if (VALID_IDX(idx)) {
        p_cellcb = GET_CELLCB(idx);
    }
    
    VAR_revolution = revolution;
}

/* #[<ENTRY_FUNC>]# eMotor_getRevolution
 * name:         eMotor_getRevolution
 * global_name:  tMotor_eMotor_getRevolution
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eMotor_getRevolution(CELLIDX idx, int32_t* revolution)
{
    CELLCB	*p_cellcb;
    if (VALID_IDX(idx)) {
        p_cellcb = GET_CELLCB(idx);
    }
    
    *revolution = VAR_revolution;
}

/* #[<ENTRY_PORT>]# eiMotorInterrupt
 * entry port: eiMotorInterrupt
 * signature:  siMotorInterrupt
 * context:    non-task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eiMotorInterrupt_quadDecode
 * name:         eiMotorInterrupt_quadDecode
 * global_name:  tMotor_eiMotorInterrupt_quadDecode
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eiMotorInterrupt_quadDecode(CELLIDX idx, uint32_t pins)
{
    CELLCB	*p_cellcb;
    if (VALID_IDX(idx)) {
        p_cellcb = GET_CELLCB(idx);
    }
    
    pins = ((pins >> motorDir[ATTR_portNumber]) & 0x01) | ((pins >> (motorEdge[ATTR_portNumber] - 1)) & 0x02);
    
    if((pins & 0x01) != VAR_preEdge){
        switch(pins){
          case 0x00:
            VAR_revolution--;
            break;
          case 0x01:
            VAR_revolution++;
            break;
          case 0x03:
            VAR_revolution--;
            break;
          case 0x02:
            VAR_revolution++;
            break;
          default:
            break;
        }
    
        VAR_preEdge = (pins & 0x01);
    }
}

/* #[<ENTRY_FUNC>]# eiMotorInterrupt_stopMotor
 * name:         eiMotorInterrupt_stopMotor
 * global_name:  tMotor_eiMotorInterrupt_stopMotor
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eiMotorInterrupt_stopMotor(CELLIDX idx)
{
    CELLCB	*p_cellcb;
    if (VALID_IDX(idx)) {
        p_cellcb = GET_CELLCB(idx);
    }
    
    cAvrMotor_setMotor(ATTR_portNumber, 0, 1);
}

/* #[<POSTAMBLE>]#
 *   艺ɔ󂯌֐܂
 * #[</POSTAMBLE>]#*/
